/*
 * service.h
 *
 * Represents the service layer (facade of the domain).
 *
 * The system events are handled here (controller operations) - e.g.
 * "create task", "get tasks".
 *
 *  Created on: Mar 21, 2015
 *      Author: ilazar
 */

#ifndef SERVICE_H_
#define SERVICE_H_

#include "domain.h"
#include "repository.h"
#include "utils/debug.h"
#include <string>
#include <exception>
#include <list>

namespace Todo {

class TodoManager {
public:
	/**
	 * Initializes a todo manager.
	 */
	TodoManager(Repository<Task>& repository) :
			_task_repository(repository) {
		debug("TodoManager(Repository<Task>&)");
		_tasks = _task_repository.find_all();
	}

	/**
	 * Destroys the cached tasks.
	 */
	~TodoManager() {
		debug("~TodoManager()");
		if (_tasks != 0) {
			delete _tasks;
		}
	}

	/**
	 * Creates a new ACTIVE Task having a given text.
	 *
	 * If the operation succeeds, the new task is returned,
	 * otherwise logic_error in case the text is not valid, or
	 * runtime_error in case of unknown I/O errors.
	 */
	const Task& create_task(const string& text) const throw (logic_error, runtime_error);

	/**
	 * Returns the task list managed by a todo manager.
	 *
	 * Throws runtime_errors in case I/O errors occur.
	 */
	const list<Task>& get_tasks() throw (runtime_error);

private:
	Repository<Task>& _task_repository;
	struct TaskComparator {
		bool operator()(Task& t1, Task& t2) {
			return t1 < t2;
		}
	} task_comparator;
	list<Task>* _tasks;
};

}
#endif /* SERVICE_H_ */
